<?php
/**
 * Doctrine
 */
require_once "vendor/autoload.php";

use Doctrine\ORM\Tools\Setup;
use Doctrine\ORM\EntityManager;


// Entityの定義
/**
 * @Entity @Table(name="prefecture")
 **/
class Region
{
    /** @Id @Column(type="integer") @GeneratedValue **/
    protected $id;
    /** @Column(type="string") **/
    protected $name;

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }
    public function setName(string $name)
    {
        $this->name = $name;
    }
}

/**
 * @Entity @Table(name="prefecture")
 **/
class Prefecture
{
    /** @Id @Column(type="integer") @GeneratedValue **/
    protected $id;
    /** @Column(type="string") **/
    protected $name;
    /**
     * Regioonクラスをregion_idで参照
     * @ManyToOne(targetEntity="Region") 
     * @JoinColumn(name="region_id", referencedColumnName="id")
     */
    private $region;
    public function getId()
    {
        return $this->id;
    }
    public function getName()
    {
        return $this->name;
    }
    public function setName(string $name)
    {
        $this->name = $name;
    }
    public function getRegion()
    {
        return $this->region;
    }
    public function setRegion(Region $region)
    {
        $this->region = $region;
    }
}

// 事前準備
$config = Setup::createAnnotationMetadataConfiguration([], true);
$connectionParams = [
    'url' => 'mysql://demo:demo@127.0.0.1/demo',
];
$conn = \Doctrine\DBAL\DriverManager::getConnection($connectionParams);

$entityManager = EntityManager::create($conn, $config);

// 発行されるSQLの確認のための準備
$stack = new \Doctrine\DBAL\Logging\DebugStack();
$entityManager->getConnection()->getConfiguration()->setSQLLogger($stack);

// リポジトリからデータを取得する
$repository = $entityManager->getRepository('Prefecture');
// 全件取得
$prefectures = $repository->findAll();

// 特定のIDだけ取得しデータを更新
$prefecture = $repository->findOneById(30);
echo $prefecture->getName(); // 和歌山県
$prefecture->setName('超和歌山県');
$entityManager->flush(); // update文発行

// INSERT
/**
 * @Entity @Table(name="user")
 **/
class User
{
    /** @Id @Column(type="integer") @GeneratedValue **/
    protected $id;
    /** @Column(type="string") **/
    protected $name;

    public function getId()
    {
        return $this->id;
    }

    public function getName()
    {
        return $this->name;
    }
    public function setName(string $name)
    {
        $this->name = $name;
    }
}

$user = new User();
$user->setName('前田');
$entityManager->persist($user);
$entityManager->flush();

// N+1対応をDQLで行う
$query = $entityManager->createQuery(
  'SELECT p, r FROM Prefecture p join p.region r WHERE r.id = 2'
  );
$prefectures = $query->getResult();

// 発行されたSQLを確認
print_r($stack);

